@@ -78,6 +78,9 @@ outline_window_expose (GtkWidget      *widget,
 {
   MetaTabPopup *popup;
   TabEntry *te;  
+  GtkStyle *style;
+  GdkWindow *window;
+  cairo_t *cr;
   
   popup = data;
 
@@ -85,20 +88,26 @@ outline_window_expose (GtkWidget      *widget,
     return FALSE;
 
   te = popup->current_selected_entry;
+  window = gtk_widget_get_window (widget);
+  style = gtk_widget_get_style (widget);
+  cr = gdk_cairo_create (window);
   
-  gdk_draw_rectangle (widget->window,
-                      widget->style->white_gc,
-                      FALSE,
-                      0, 0,
-                      te->rect.width - 1,
-                      te->rect.height - 1);
+  cairo_set_line_width (cr, 1.0);
+  gdk_cairo_set_source_color (cr, &style->white);
 
-  gdk_draw_rectangle (widget->window,
-                      widget->style->white_gc,
-                      FALSE,
-                      te->inner_rect.x - 1, te->inner_rect.y - 1,
-                      te->inner_rect.width + 1,
-                      te->inner_rect.height + 1);
+  cairo_rectangle (cr,
+                   0.5, 0.5,
+                   te->rect.width - 1,
+                   te->rect.height - 1);
+  cairo_stroke (cr);
+
+  cairo_rectangle (cr,
+                   te->inner_rect.x - 0.5, te->inner_rect.y - 0.5,
+                   te->inner_rect.width + 1,
+                   te->inner_rect.height + 1);
+  cairo_stroke (cr);
+
+  cairo_destroy (cr);
 
   return FALSE;
 }
@@ -437,7 +446,7 @@ meta_ui_tab_popup_set_showing (MetaTabPopup *popup,
         {
           meta_verbose ("Hiding tab popup window\n");
           gtk_widget_hide (popup->window);
-          meta_core_increment_event_serial (gdk_display);
+          meta_core_increment_event_serial (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
         }
     }
 }
@@ -470,7 +479,7 @@ display_entry (MetaTabPopup *popup,
     {
       /* Do stuff behind gtk's back */
       gdk_window_hide (popup->outline_window->window);
-      meta_core_increment_event_serial (gdk_display);
+      meta_core_increment_event_serial (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
   
       rect = te->rect;
       rect.x = 0;
@@ -591,7 +600,7 @@ meta_ui_tab_popup_select (MetaTabPopup *popup,
 }
 
 #define META_TYPE_SELECT_IMAGE            (meta_select_image_get_type ())
-#define META_SELECT_IMAGE(obj)            (GTK_CHECK_CAST ((obj), META_TYPE_SELECT_IMAGE, MetaSelectImage))
+#define META_SELECT_IMAGE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), META_TYPE_SELECT_IMAGE, MetaSelectImage))
 
 typedef struct _MetaSelectImage       MetaSelectImage;
 typedef struct _MetaSelectImageClass  MetaSelectImageClass;
@@ -644,7 +653,7 @@ static GtkImageClass *parent_class;
 GType
 meta_select_image_get_type (void)
 {
-  static GtkType image_type = 0;
+  static GType image_type = 0;
 
   if (!image_type)
     {
@@ -672,7 +681,7 @@ meta_select_image_class_init (MetaSelectImageClass *klass)
 {
   GtkWidgetClass *widget_class;
   
-  parent_class = gtk_type_class (gtk_image_get_type ());
+  parent_class = g_type_class_peek (gtk_image_get_type ());
 
   widget_class = GTK_WIDGET_CLASS (klass);
   
@@ -687,6 +696,9 @@ meta_select_image_expose_event (GtkWidget      *widget,
     {
       int x, y, w, h;
       GtkMisc *misc;
+      GtkStyle *style;
+      GtkStateType state;
+      cairo_t *cr;
 
       misc = GTK_MISC (widget);
       
@@ -705,33 +717,37 @@ meta_select_image_expose_event (GtkWidget      *widget,
       w = widget->requisition.width - OUTSIDE_SELECT_RECT * 2 - 1;
       h = widget->requisition.height - OUTSIDE_SELECT_RECT * 2 - 1;
 
-      gdk_draw_rectangle (widget->window,
-                          widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
-                          FALSE,
-                          x, y, w, h);
-      gdk_draw_rectangle (widget->window,
-                          widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
-                          FALSE,
-                          x - 1, y - 1, w + 2, h + 2);
-      
+      style = gtk_widget_get_style (widget);
+      state = gtk_widget_get_state (widget);
+      cr = gdk_cairo_create (widget->window);
+
+      cairo_set_line_width (cr, 2.0);
+      gdk_cairo_set_source_color (cr, &style->fg[state]);
+
+      cairo_rectangle (cr, x, y, w + 1, h + 1);
+      cairo_stroke (cr);
+
+      cairo_set_line_width (cr, 1.0);
 #if 0
-      gdk_draw_rectangle (widget->window,
-                          widget->style->bg_gc[GTK_STATE_SELECTED],
-                          TRUE,
-                          x, y, w, h);
+      gdk_cairo_set_source_color (cr, &style->bg[GTK_STATE_SELECTED]);
+      cairo_rectangle (cr, x, y, w, h);
+      cairo_fill (cr);
 #endif
+
 #if 0      
       gtk_paint_focus (widget->style, widget->window,
                        &event->area, widget, "meta-tab-image",
                        x, y, w, h);
 #endif
+
+      cairo_destroy (cr);
     }
 
   return GTK_WIDGET_CLASS (parent_class)->expose_event (widget, event);
 }
 
 #define META_TYPE_SELECT_WORKSPACE   (meta_select_workspace_get_type ())
-#define META_SELECT_WORKSPACE(obj)   (GTK_CHECK_CAST ((obj), META_TYPE_SELECT_WORKSPACE, MetaSelectWorkspace))
+#define META_SELECT_WORKSPACE(obj)   (G_TYPE_CHECK_INSTANCE_CAST ((obj), META_TYPE_SELECT_WORKSPACE, MetaSelectWorkspace))
 
 typedef struct _MetaSelectWorkspace       MetaSelectWorkspace;
 typedef struct _MetaSelectWorkspaceClass  MetaSelectWorkspaceClass;
@@ -797,7 +813,7 @@ static gboolean meta_select_workspace_expose_event (GtkWidget      *widget,
 GType
 meta_select_workspace_get_type (void)
 {
-  static GtkType workspace_type = 0;
+  static GType workspace_type = 0;
 
   if (!workspace_type)
     {
@@ -873,6 +889,8 @@ meta_select_workspace_expose_event (GtkWidget      *widget,
 {
   MetaWorkspace *workspace;
   WnckWindowDisplayInfo *windows;
+  GtkStyle *style;
+  cairo_t *cr;
   int i, n_windows;
   GList *tmp, *list;
 
@@ -928,19 +946,20 @@ meta_select_workspace_expose_event (GtkWidget      *widget,
   
   if (META_SELECT_WORKSPACE (widget)->selected)
     {
-      i = SELECT_OUTLINE_WIDTH - 1;
-      while (i >= 0)
-        {
-          gdk_draw_rectangle (widget->window,
-                              widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
-                              FALSE,
-                              i, 
-                              i,
-                              widget->allocation.width - i * 2 - 1,
-                              widget->allocation.height - i * 2 - 1);
+      style = gtk_widget_get_style (widget);
+      cr = gdk_cairo_create (widget->window);
 
-          --i;
-        }
+      gdk_cairo_set_source_color (cr,
+                                  &style->fg[gtk_widget_get_state (widget)]);
+      cairo_set_line_width (cr, SELECT_OUTLINE_WIDTH);
+
+      cairo_rectangle (cr,
+                       SELECT_OUTLINE_WIDTH / 2.0, SELECT_OUTLINE_WIDTH / 2.0,
+                       widget->allocation.width - SELECT_OUTLINE_WIDTH,
+                       widget->allocation.height - SELECT_OUTLINE_WIDTH);
+      cairo_stroke (cr);
+
+      cairo_destroy (cr);
     }
 
   return TRUE;
